<?php

namespace UBRIR;

abstract class CMS_PaymentCallback
{
    const PaymentApproved = '';

    private $gatewayPanel;
    protected $callbackDataContainer;
    protected $_PaymentStatus;

    protected $_ShopOrderId;
    protected $_ShopPaymentId;
    protected $_continueLink;
    protected $_resultData;

    function __construct(InterfaceConnectionConfig $connector = null)
    {
        $this->gatewayPanel = new GatewayInteractionPanel($connector);

        $this->callbackDataContainer = $this->HandleCallback();
    }

    public function AddResultMessage($message, $code = '')
    {
        $this->callbackDataContainer->SetResultRecord($code, $message);
    }

    public function GetShopOrderId()
    {
        if ($this->_ShopOrderId == null) {
            throw new \Exception('Не выполнена обработка запроса методом ProcessCallbackResponse');
        }

        return $this->_ShopOrderId;
    }

    public function GetPaymentId()
    {
        if ($this->_ShopPaymentId == null) {
            throw new \Exception('Не выполнена обработка запроса методом ProcessCallbackResponse');
        }

        return $this->_ShopPaymentId;
    }

    private function SetPaymentState($orderState)
    {
        if (TypeEnums::isValidOrderState($orderState)) {
            $this->_PaymentStatus = $orderState;
        } else {
            $this->_PaymentStatus = null;
        }
    }

    public function isPaymentApproved()
    {
        if ($this->_PaymentStatus == TypeEnums::OrderStatus_APPROVED) {
            return true;
        }

        return false;
    }

    public function GetPaymentStatus()
    {
        return $this->_PaymentStatus;
    }

    public function GetPaymentDescription()
    {
        return $this->callbackDataContainer->getFieldValue('OrderStatusScr');
    }

    public function GetOrderSum()
    {
        $floatSum = 0.0;

        $floatSum = (float)$this->callbackDataContainer->getFieldValue('PurchaseAmount') / 100;

        return $floatSum;
    }


    private function HandleCallback()
    {
        $this->gatewayPanel->Log('Обработка callback от шлюза');

        if (!isset($_POST['xmlmsg'])) {
            $response = new DataContainer();
            $response->AddError('Неверный запрос сервера');
        }

        $XMLString = base64_decode($_POST['xmlmsg']);
        $xmlLines = explode("\n", $XMLString);

        //Проверяем что с XML все в порядке (по большому счету это нужно только как защита от прямых дерганий callback
        libxml_use_internal_errors(); //Подавляем вывод ошибок XML
        simplexml_load_string($XMLString);
        $xmlErrors = libxml_get_errors();
        if (count($xmlErrors) > 0) {
            $this->gatewayPanel->Log('Обнаружены ошибки в XML');
            foreach ($xmlErrors as $error) {
                $this->gatewayPanel->Log(self::FormatXMLError($error, $xmlLines));
            }

            $response = new DataContainer();
            $response->AddError('Обнаружены ошибки в XML');
            $this->gatewayPanel->LogErrors($response);
        } else {
            $this->gatewayPanel->Log('Callback: Разбирем полученный XML');
            $this->gatewayPanel->Log($XMLString);

            $objXML = simplexml_load_string($XMLString);
            $arKeyValueXML = ServerConnector::XML2Array($XMLString);
            $response = new DataContainer();
            $response->setSuccess()->SaveXMLData($XMLString, $objXML, $arKeyValueXML);
        }

        return $response;
    }

    public function isRequestSuccessfullyParsed()
    {
        return $this->callbackDataContainer->isSuccess();
    }

    public function ProcessCallbackResponse()
    {
        if ($this->callbackDataContainer->isSuccess()) {
            $orderStatus = $this->callbackDataContainer->getFieldValue('OrderStatus');
            $TWPG_OrderID = $this->callbackDataContainer->getFieldValue('OrderId');

            $dbRow = $this->gatewayPanel->GetOrderDataByUBRIROrderID($TWPG_OrderID);
            if ($dbRow === false) {
                //TODO: только обработка, выводом шаблонов и ветвлением по ним должен заняться ShowCallbackProcessResult
                $this->gatewayPanel->ShowPaymentError(
                    false,
                    'Не найден заказ с платежным идентификатором = '.$TWPG_OrderID
                );

                return;
            }

            $this->_ShopOrderId = intval($dbRow['order_id']);
            $this->_ShopPaymentId = intval($dbRow['payment_id']);
            $SessionId = $dbRow['session_id'];

            $this->_resultData = new PaymentResultViewer();
            $this->_resultData
                ->SetPaymentTime($this->callbackDataContainer->getFieldValue('date'))
                ->SetShopOrderId($this->_ShopOrderId)
                ->SetTWPG_OrderId($TWPG_OrderID)
                ->SetCardPAN($this->callbackDataContainer->getFieldValue('PAN'))
                ->SetCardOwner($this->callbackDataContainer->getFieldValue('Name'))
                ->SetPaymentAmount($this->callbackDataContainer->getFieldValue('PurchaseAmountScr'));

            $this->SetPaymentState($orderStatus);

            switch ($orderStatus) {
                case TypeEnums::OrderStatus_APPROVED:
                    {

                        $checkOrderStatusResponse = $this->gatewayPanel->SetupAndCheckOrderStatus(
                            $TWPG_OrderID,
                            $SessionId
                        );

                        if (!$checkOrderStatusResponse->isSuccess()) {
                            $bankResponseDescription = $checkOrderStatusResponse->getFieldValue('ResponseDescription');

                            $this->_resultData->AddExtraMessage($bankResponseDescription, 'Сообщение');
                        } else {
                            $this->_continueLink = $this->OnPaymentApproved($this->_ShopOrderId, $SessionId);

                            //Отправка почтового уведомления
                            $mailNotifier = $this->gatewayPanel->GetCMSConnector()->GetMailNotifier();
                            if ($mailNotifier !== null) {
                                $mailNotifier->ConnectResultViewer($this->_resultData);

                                if (!$mailNotifier->SendPaymentNotification()) {

                                    $mailErrorData = new DataContainer();

                                    $receiverEmail = $mailNotifier->GetDefaultReceiversEmail();
                                    $mailErrorData->AddError(
                                        'Не удалось отправить почтовое уведомление! Email получателя: '.$receiverEmail.'. Проверьте правильность email администратора магазина и возможность отправки почты с сервера'
                                    );
                                    $this->gatewayPanel->LogErrors($mailErrorData);
                                }
                            }
                        }
                    }
                    break;
                case TypeEnums::OrderStatus_CANCELED:
                    {
                        $this->_resultData->SetMessageBaseOnResponse($this->callbackDataContainer);
                        $this->_continueLink = $this->OnPaymentCanceled($this->_ShopOrderId, $SessionId);
                    }
                    break;
                case TypeEnums::OrderStatus_DECLINED:
                    {
                        $this->_resultData->SetMessageBaseOnResponse($this->callbackDataContainer);

                        $this->_continueLink = $this->OnPaymentDeclined($this->_ShopOrderId, $SessionId);
                    }
                    break;
                default:
                    {
                        $this->gatewayPanel->Log('Не удалось определить OrderStatus!');
                    }
                    break;
            }
        }
    }

    public function ShowCallbackProcessResult()
    {
        if ($this->_PaymentStatus == null) {
            $this->ProcessCallbackResponse();
        }

        if (!is_string($this->_continueLink)) {
            $this->_continueLink = $this->gatewayPanel->GetCMSConnector()->GetDefaultContinueLink();
        }

        if ($this->callbackDataContainer->isSuccess()) {

            switch ($this->_PaymentStatus) {
                case TypeEnums::OrderStatus_APPROVED:
                    {
                        $successMessage = self::GetPaymentDescription();

                        $this->gatewayPanel->ShowPaymentSuccess(
                            $this->_continueLink,
                            $successMessage,
                            $this->_resultData
                        );

                        return true;
                    }
                    break;

                case TypeEnums::OrderStatus_CANCELED:
                    {
                        $this->gatewayPanel->ShowPaymentError($this->_continueLink, '', $this->_resultData);

                        return false;
                    }
                    break;

                case TypeEnums::OrderStatus_DECLINED:
                    {
                        $this->gatewayPanel->ShowPaymentError($this->_continueLink, '', $this->_resultData);

                        return false;
                    }
                    break;
            }

            $this->gatewayPanel->ShowPaymentError(false, '', $this->_resultData);
        } else {
            $this->gatewayPanel->ShowPaymentError(false, 'Неверный формат ответа');
        }
    }

    private function FormatXMLError($error, $xml)
    {
        $return = $xml[$error->line - 1]."\n";
        $return .= str_repeat('-', $error->column)."^\n";

        switch ($error->level) {
            case LIBXML_ERR_WARNING:
                $return .= "Warning $error->code: ";
                break;
            case LIBXML_ERR_ERROR:
                $return .= "Error $error->code: ";
                break;
            case LIBXML_ERR_FATAL:
                $return .= "Fatal Error $error->code: ";
                break;
        }

        $return .= trim($error->message).
            "\n  Line: $error->line".
            "\n  Column: $error->column";

        if ($error->file) {
            $return .= "\n  File: $error->file";
        }

        return "$return\n\n--------------------------------------------\n\n";
    }


    /**
     * @param $shopOrderId
     * @param $sessionId
     * @return string
     */
    abstract public function OnPaymentApproved($shopOrderId, $sessionId);

    abstract public function OnPaymentCanceled($shopOrderId, $sessionId);

    abstract public function OnPaymentDeclined($shopOrderId, $sessionId);
}